package rrdService.service;
import rrdService.util.*;
import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.registry.*;
import java.net.*;
import java.io.*;
import java.lang.*;
import java.util.*;
/**	
* The class RoundRobin allows to share methodes between RoundRobinLibrary and RoundRobinDataBase
* @author	Damien Pellier
* @version	1.0
* date : 15/07/2001
* </code>
* @see <a href="RoundRobin.java">RoundRobin</a>
* @see RoundRobinItf RoundRobinLibraryItf RoundRobinDataBaseItf
*/
public abstract class RoundRobin extends UnicastRemoteObject implements RoundRobinItf, LockItf {
	/**
	* Type RoundRobin LIBRARY
	*/
	protected static final String RRD_LIBRARY = "LIBRARY";
	/**
	* Type RoundRobin FILE
	*/
	protected static final String RRD_FILE = "FILE";
	/**
	* Extention of mountFile rrd
	*/
	public static final String RRD_FILE_EXTENTION = ".RRD";
	/**
	* File representing the RoundRobin
	*/
	protected File mountFile;
	/**
	* The type of the RoundRobin
	*/
	protected String type;
	/**
	* Constructor of class RoundRobin
	*/
	public RoundRobin(String name, String type) throws Exception {
		super();
		this.setMountFile(name);
		this.setType(type);
	}
	/**
	* getAbsolutePath return the ablosulte path of RoundRobin mount file
	* @return String representing the ablosulte path of RoundRobin mount file
	*/
	protected String getAbsolutePath() {
		return mountFile.getAbsolutePath();
	}
	/**
	* setMountFile specify the mount file of RoundRobin
	* @param nameOfMountFile the name of the mount file
	*/
	protected void setMountFile(String nameOfMountFile) { 
		this.mountFile = new File(nameOfMountFile);
	}
	/**
	* getMountFile return the mount file of RoundRobin
	* @return the mount file of RoundRobin
	*/
	protected File getMountFile() {
		return this.mountFile;
	}
	/**
	* setLock specify the lock
	* @param lock the lock to associate to the RoundRobin
	*/
	protected void setLock(Lock lock) {
		RoundRobinService.lockTable.putLock(getAbsolutePath(), lock);
	}
	/**
	* getLock return the lock of RoundRobin
	* @return Lock the lock of RoundRobin
	*/
	protected Lock getLock() throws Exception {
		return RoundRobinService.lockTable.getLock(getAbsolutePath());
	}
	/**
	* setType specify the type of a RoundRobin
	* @param type the type of RoundRobin
	*/
	protected void setType(String type) throws Exception {
		type = type.toUpperCase();
		if (!isValideType(type)) throwException("Error type not match", null);
		this.type = type;
	}
	/**
	* getName return the name of RoundRobin
	*/
	public String getName() throws RemoteException {
		String name = new String();
		try {
			this.lockRead();
			name = getMountFile().getName();
			this.unLockRead();
		} catch(Exception e) {
			throwRemoteException("Error getting name", e);
		}
		return name;
	}
	/**
	* getType return the type of a RoundRobin
	*/
	public String getType() throws RemoteException {
		try {
			this.lockRead();
			String type = this.type;
			this.unLockRead();
		} catch(Exception e) {
			throwRemoteException("Error getting Type of RoundRobin", e);
		}
		return type;
	}
	/**
	* isRoundRobinLibrary return true if RoundRobin is a RoundRobinlibrary
	*/
	public boolean isRoundRobinLibrary() throws RemoteException {
		boolean result =  false;
		try {
			this.lockRead();
			result = this.getType().equals(RRD_LIBRARY);
			this.unLockRead();
		} catch(Exception e) {
			throwRemoteException("Error testing if RoundRobin is a library", e);
		}
		return result;
	}
	/**
	* isRoundRobinDataBase return true if RoundRobin is a RoundRobinDataBase
	*/
	public boolean isRoundRobinDataBase() throws RemoteException {
		boolean result =  false;
		try {
			this.lockRead();
			result = this.getType().equals(RRD_FILE);
			this.unLockRead();
		} catch(Exception e) {
			throwRemoteException("Error testing if RoundRobin is a RoundRobinDataBase", e);
		}
		return result;
	}
	/**
	* isRoundRobinDataBase return true if RoundRobin is a RoundRobinDataBase
	*/
	protected boolean isRoundRobinDataBase(File mountFile) {
		String fileName = mountFile.getName();
		if (fileName.lastIndexOf(".") == -1) return false;
		return  fileName.substring(fileName.lastIndexOf("."), fileName.length()).equalsIgnoreCase(RRD_FILE_EXTENTION);
	}
	/**
	* isRoundRobinLibrary return true if RoundRobin is a RoundRobinlibrary
	*/
	protected boolean isRoundRobinLibrary(File mountFile) {
		return mountFile.isDirectory();
	}
	/**
	* isValideType return true if the type is valide
	* @param type the type of RoundRobin to test
	*/
	protected boolean isValideType(String type) {
		if (!type.equals(RRD_LIBRARY) && !type.equals(RRD_FILE)) return false;
		else return true;
	}
	/**
	* Lock in reading acces
	* return boolean true if the lock is taken
	*/
	public boolean lockRead() throws Exception {
		return this.getLock().lockRead();
	}
	/**
	* Lock in writting acces
	* return boolean true if the lock is taken
	*/
	public boolean lockWrite() throws Exception {
		return this.getLock().lockWrite();
	}
	/**
	* Lock in deleting acces
	* return boolean true if the lock is taken
	*/
	public boolean lockDelete() throws Exception {
		return this.getLock().lockDelete();
	}
	/**
	* Unlock in reading acces
	* return boolean true if the lock is left
	*/
	public boolean unLockRead() throws Exception {
		return this.getLock().unLockRead();
	}
	/**
	* Unlock in writting acces
	* return boolean true if the lock is left
	*/
	public boolean unLockWrite() throws Exception {
		return this.getLock().unLockWrite();
	}
	/**
	* getStatus return the status of lock
	* return int reprsenting the status READ_WRITE or DELETE
	*/
	public int getStatus() throws Exception {
		return this.getLock().getStatus();
	}
	/**
	* getSize return the size of RoundRobin
	* @return long
	*/
	public abstract long getSize() throws RemoteException;
	/**
	* delete delete a rrd
	* @return int the number of rrd delete
	*/
	public abstract int delete() throws RemoteException;
	
	/**
	* throwRemoteException throw a remote exception with the specific message
	* @param message the exception message
	* @param e the exception which occurs
	*/
	protected void throwRemoteException(String message, Exception e) throws RemoteException {
		if (e.getMessage() != null ) {
			LogFile.print(message+": "+e.getMessage());
			throw new RemoteException(message+": "+e.getMessage());
		}
		else {
			LogFile.print(message);
			throw new RemoteException(message);
		}
	}
	/**
	* throwException throw a exception with the specific message
	* @param message the exception message
	* @param e the exception which occurs
	*/
	protected void throwException(String message, Exception e) throws Exception {
		if (e != null) {
			if (e.getMessage() != null) {
				throw new Exception(message+": "+e.getMessage());
			}
		}
		else {
			throw new Exception(message);
		}	
	}
}	