package rrdService.service;
import rrdService.util.*;
import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.registry.*;
import java.net.*;
import java.io.*;
import java.lang.*;
import java.util.*;
import java.awt.*;
/**	
* The class RoundRobinDataBase allows the managment of a group of rrd
* @author	Damien Pellier
* @version	1.0
* date : 15/07/2001
* </code>
* @see <a href="RoundRobinDataBase.java">RoundRobinDataBase</a>
* @see RoundRobinDataBaseItf RoundRobin 
*/
public class RoundRobinDataBase extends RoundRobin implements RoundRobinDataBaseItf {
	/**
	* RRDTOOL_CMD command of rrdtool.exe
	*/
	private static final String RRDTOOL_CMD = "."+File.separator+"class"+File.separator+"rrdService"+File.separator+"rrdTool.exe";
	/**
	* RRDTOOL_CMD_BAT command of rrdtool.bat
	*/
	private static final String RRDTOOL_CMD_BAT = "."+File.separator+"class"+File.separator+"rrdService"+File.separator+"rrdTool.bat";
	/**
	* Mutex allowing the synchronisation of graph method
	*/
	private static Mutex mutex = new Mutex(1);
	/**
	* Constructor of class RoundRobinDataBase
	* @param fileName the name of the RoundRobinDataBase
	*/
	public RoundRobinDataBase(String fileName) throws Exception {
		super(fileName, RRD_FILE);
	}
	/**
	* Constructor of class RoundRobinDataBase
	* @param fileName the name of the RoundRobinDataBase
	* @param startTime Specifies the time in seconds since 1970-01-01 UTC when the first value should be added to the RRD (default null)
	* @param steps Specifies the base interval in seconds with which data will be fed into the RRD (default 0)
	* @param dataSourceContainer a collection of the data source of the RRD
	* @param roundRobinArchiveContainer a collection the RRA of the RRD
	*/
	public RoundRobinDataBase(String fileName,
								Time startTime,
								int steps,
								DataSourceContainer dataSourceContainer,
								RoundRobinArchiveContainer roundRobinArchiveContainer) throws Exception {
		super(fileName, RRD_FILE);
		if (!this.getMountFile().exists()) {
			this.setLock(new Lock());
			int cmdSize = dataSourceContainer.size()+roundRobinArchiveContainer.size()+7;
			if (startTime == null) cmdSize -=2; 
			if (steps == 0) cmdSize -=2;
			String cmd[] = new String[cmdSize];			
			cmd[0] = RRDTOOL_CMD;
			cmd[1] = "create";
			cmd[2] = fileName;
			int offset = 3;
			if (startTime != null) {
				cmd[offset++] = "--start";
				cmd[offset++] = startTime.toString();
			}
			if (steps != 0) {
				cmd[offset++] = "--step";
				cmd[offset++] = String.valueOf(steps);
			}
			for(int i=0; i<dataSourceContainer.size(); i++) {
				cmd[offset++] = dataSourceContainer.dataSourceAt(i).toString();
			}
			for(int i=0; i<roundRobinArchiveContainer.size(); i++) {
				cmd[offset++] = roundRobinArchiveContainer.roundRobinArchiveAt(i).toString();
			}
			rrdToolCmd(cmd);
		}
	}
	/**
	* Update update a round robin database
	* @param template The template switch allows you to specify which data sources you are going to update and in which order. 
	* @param dataContainer The data used for updating the RRD was acquired at a certain time. 
	*/
	public void update(Template template, DataContainer dataContainer) throws RemoteException {
		try {
			if (!this.getMountFile().exists()) 
				throwException("round robine database doesn't exist or was removed", null);
			this.lockWrite();
			int cmdSize = 4;
			if (template != null) cmdSize += template.size(); 
			String cmd[] = new String[cmdSize];			
			cmd[0] = RRDTOOL_CMD;
			cmd[1] = "update";
			cmd[2] = this.getAbsolutePath();
			int offset = 3;
			if (template != null) {
				cmd[offset++] = "--template";
				cmd[offset++] = template.toString();
			}
			int index = offset++;
			cmd[index] = new String(dataContainer.dataAt(0).getTimestamp());
			for(int i=0; i<dataContainer.size(); i++) {
				cmd[index] += ":"+dataContainer.dataAt(i).getValue();
			}
			rrdToolCmd(cmd);
			this.unLockWrite();
		} catch(Exception e) {
			throwRemoteException("Error updating RoundRobinDataBase "+this.getMountFile().getName(), e);
		}
	}
	/**
	* Update update a round robin database
	* @param dataContainer The data used for updating the RRD was acquired at a certain time. 
	*/
	public void update(DataContainer dataContainer) throws RemoteException {
		update(null, dataContainer);
	}
	/**
	* Update update a round robin database
	* @param data The data used for updating the RRD was acquired at a certain time. 
	*/
	public void update(Data data) throws RemoteException {
		DataContainer container = new DataContainer();
		container.addElement(data);
		update(null, container);
	}
	/**
	* fetch fetch a round robin database
	* @param cf which consolidation function should have been applied to the data you want to fetch? (AVERAGE,MIN,MAX,LAST) 
	* @param resolution what interval should the values have (seconds per value). rrdfetch will try to match your request, but it will return data even if no absolute match is possible. 
	* @param startTime when should the data begin. A time in seconds since epoch (1970-01-01) is required. Negative numbers are relative to the current time. By default one day worth of data will be fetched. See also AT-STYLE TIME SPECIFICATION section for a detailed explanation on ways to specify start time. 
	* @param endTime when should the data end. Time in seconds since epoch. See also AT-STYLE TIME SPECIFICATION section for a detailed explanation of how to specify end time. 
	*/
	public String fetch(String cf, int resolution, Time startTime, Time endTime) throws RemoteException {
		String result = new String();
		try {
			if (!this.getMountFile().exists()) 
				throwException("round robine database doesn't exist or was removed", null);
			this.lockRead();
			int cmdSize = 10;
			if (resolution == -1) cmdSize -= 2; 
			if (startTime == null) cmdSize -= 2;
			if (endTime == null) cmdSize -= 2;
			String cmd[] = new String[cmdSize];			
			cmd[0] = RRDTOOL_CMD;
			cmd[1] = "fetch";
			cmd[2] = this.getAbsolutePath();
			cmd[3] = cf;
			int offset = 4;
			if (resolution != -1) {
				cmd[offset++] = "--resolution";
				cmd[offset++] = String.valueOf(resolution);
			}
			if (startTime != null) {
				cmd[offset++] = "--start";
				cmd[offset++] = startTime.toString();
			}
			if (endTime != null) {
				cmd[offset++] = "--end";
				cmd[offset++] = endTime.toString();
			}
			result = rrdToolCmd(cmd);
			this.unLockRead();
		} catch(Exception e) {
			throwRemoteException("Error fetching RoundRobinDataBase "+this.getMountFile().getName(), e);
		}
		return result;
	}
	/**
	* fetch fetch a round robin database
	* @param cf which consolidation function should have been applied to the data you want to fetch? (AVERAGE,MIN,MAX,LAST) 
	* @param startTime when should the data begin. A time in seconds since epoch (1970-01-01) is required. Negative numbers are relative to the current time. By default one day worth of data will be fetched. See also AT-STYLE TIME SPECIFICATION section for a detailed explanation on ways to specify start time. 
	* @param endTime when should the data end. Time in seconds since epoch. See also AT-STYLE TIME SPECIFICATION section for a detailed explanation of how to specify end time. 
	*/
	public String fetch(String cf, Time startTime, Time endTime) throws RemoteException {
		return fetch(cf, -1, startTime, endTime);
	}
	/**
	* fetch fetch a round robin database
	* @param cf which consolidation function should have been applied to the data you want to fetch? (AVERAGE,MIN,MAX,LAST) 
	* @param resolution what interval should the values have (seconds per value). rrdfetch will try to match your request, but it will return data even if no absolute match is possible. 
	* @param startTime when should the data begin. A time in seconds since epoch (1970-01-01) is required. Negative numbers are relative to the current time. By default one day worth of data will be fetched. See also AT-STYLE TIME SPECIFICATION section for a detailed explanation on ways to specify start time. 
	*/
	public String fetch(String cf, int resolution, Time startTime) throws RemoteException {
		return fetch(cf, resolution, startTime);
	}
	/**
	* fetch fetch a round robin database
	* @param cf which consolidation function should have been applied to the data you want to fetch? (AVERAGE,MIN,MAX,LAST) 
	* @param startTime when should the data begin. A time in seconds since epoch (1970-01-01) is required. Negative numbers are relative to the current time. By default one day worth of data will be fetched. See also AT-STYLE TIME SPECIFICATION section for a detailed explanation on ways to specify start time. 
	*/
	public String fetch(String cf, Time startTime) throws RemoteException {
		return fetch(cf, -1, startTime, null);
	}
	/**
	* fetch fetch a round robin database
	* @param cf which consolidation function should have been applied to the data you want to fetch? (AVERAGE,MIN,MAX,LAST) 
	* @param resolution what interval should the values have (seconds per value). rrdfetch will try to match your request, but it will return data even if no absolute match is possible. 
	* @param startTime when should the data begin. A time in seconds since epoch (1970-01-01) is required. Negative numbers are relative to the current time. By default one day worth of data will be fetched. See also AT-STYLE TIME SPECIFICATION section for a detailed explanation on ways to specify start time. 
	*/
	public String fetch(String cf, int resolution) throws RemoteException {
		return fetch(cf, resolution, null, null);
	}
	/**
	* fetch fetch a round robin database
	* @param cf which consolidation function should have been applied to the data you want to fetch? (AVERAGE,MIN,MAX,LAST) 
	*/
	public String fetch(String cf) throws RemoteException {
		return fetch(cf, -1, null, null);
	}
	/**
	* last return the date of the last data sample in an RRD
	*/
	public Time last() throws RemoteException {
		Time lastTime = null;
		try {
			if (!this.getMountFile().exists()) 
				throwException("round robine database doesn't exist or was removed", null);
			this.lockRead();
			String cmd[] = new String[3];			
			cmd[0] = RRDTOOL_CMD;
			cmd[1] = "last";
			cmd[2] = this.getAbsolutePath();
			String result = rrdToolCmd(cmd);
			result = result.substring(0, 9);
			lastTime = new Time(Long.parseLong(result), RoundRobinService.timeZone);
			this.unLockRead();
		} catch(Exception e) {
			e.printStackTrace();
			throwRemoteException("Error getting last update about RoundRobinDataBase "+this.getMountFile().getName(), e);
		}
		return lastTime;
	}
	/**
	* setHeartbeat modify the heartbeat of this RRD
	* @param dsName the ds-name of data source of this RRD to modify
	* @param heartbeat the new heartbeat of this RRD
	*/
	public void setHeartbeat(String dsName, int heartbeat) throws RemoteException {
		try {
			if (!this.getMountFile().exists()) 
				throwException("round robine database doesn't exist or was removed", null);
			this.lockWrite();
			String cmd[] = new String[5];			
			cmd[0] = RRDTOOL_CMD;
			cmd[1] = "tune";
			cmd[2] = this.getAbsolutePath();
			cmd[3] = "--heartbeat";
			cmd[4] = dsName+":"+String.valueOf(heartbeat);
			rrdToolCmd(cmd);
			this.unLockWrite();
		} catch(Exception e) {
			throwRemoteException("Error setting heartbeat of RoundRobinDataBase "+this.getMountFile().getName(), e);
		}
	}
	/**
	* setMinimum modify the minimum of this RRD
	* @param dsName the ds-name of data source of this RRD to modify
	* @param minimum the new minimum of this RRD
	*/
	public void setMinimum(String dsName, int minimum) throws RemoteException {
		try {
			if (!this.getMountFile().exists()) 
				throwException("round robine database doesn't exist or was removed", null);
			this.lockWrite();
			String cmd[] = new String[5];			
			cmd[0] = RRDTOOL_CMD;
			cmd[1] = "tune";
			cmd[2] = this.getAbsolutePath();
			cmd[3] = "--minimum";
			cmd[4] = dsName+":"+String.valueOf(minimum);
			rrdToolCmd(cmd);
			this.unLockWrite();
		} catch(Exception e) {
			throwRemoteException("Error setting minimum of RoundRobinDataBase "+this.getMountFile().getName(), e);
		}
	}
	/**
	* setMaximum modify the maximum of this RRD
	* @param dsName the ds-name of data source of this RRD to modify
	* @param maximum the new maximum of this RRD
	*/
	public void setMaximum(String dsName, int maximum) throws RemoteException {
		try {
			if (!this.getMountFile().exists()) 
				throwException("round robine database doesn't exist or was removed", null);
			this.lockWrite();
			String cmd[] = new String[5];			
			cmd[0] = RRDTOOL_CMD;
			cmd[1] = "tune";
			cmd[2] = this.getAbsolutePath();
			cmd[3] = "--maximum";
			cmd[4] = dsName+":"+String.valueOf(maximum);
			rrdToolCmd(cmd);
			this.unLockWrite();
		} catch(Exception e) {
			throwRemoteException("Error setting maximum of RoundRobinDataBase "+this.getMountFile().getName(), e);
		}
	}
	/**
	* setDataSourceType modify the type of data source of this RRD
	* @param dsName the ds-name of data source of this RRD to modify
	* @param dataSourceType the new type of data source of this RRD
	*/
	public void setDataSourceType(String dsName, String dataSourceType) throws RemoteException {
		try {
			if (!DataSource.isValideDataSourceType(dataSourceType))
				throwException("Data source type not match", null);
			if (!this.getMountFile().exists()) 
				throwException("round robine database doesn't exist or was removed", null);
			this.lockWrite();
			String cmd[] = new String[5];			
			cmd[0] = RRDTOOL_CMD;
			cmd[1] = "tune";
			cmd[2] = this.getAbsolutePath();
			cmd[3] = "--data-source-type";
			cmd[4] = dsName+":"+dataSourceType.toUpperCase();
			rrdToolCmd(cmd);
			this.unLockWrite();
		} catch(Exception e) {
			throwRemoteException("Error setting data source type of RoundRobinDataBase "+this.getMountFile().getName(), e);
		}
	}
	/**
	* setDataSourceName modify the name of data source of this RRD
	* @param oldDsName the ds-name of data source of this RRD to modify
	* @param newDsName the new name of data source of this RRD
	*/
	public void setDataSourceName(String oldDsName, String newDsName) throws RemoteException {
		try {
			if (!this.getMountFile().exists()) 
				throwException("round robine database doesn't exist or was removed", null);
			this.lockWrite();
			String cmd[] = new String[5];			
			cmd[0] = RRDTOOL_CMD;
			cmd[1] = "tune";
			cmd[2] = this.getAbsolutePath();
			cmd[3] = "--data-source-rename";
			cmd[4] = oldDsName+":"+newDsName;
			rrdToolCmd(cmd);
			this.unLockWrite();
		} catch(Exception e) {
			throwRemoteException("Error renaming data source name of RoundRobinDataBase "+this.getMountFile().getName(), e);
		}
	}
	/**
	* dump dump a round robin database
	* The dump function prints the contents of an RRD in human readable (?) XML format. This format can be read by rrdrestore. Together they allow you to transfer your files from one architecture to another as well as manipulating the contents of an RRD file in a somewhat more convenient manner.
	* @param xmlFilename the name of the output xlm file 
	*/
	public File dump(String xmlFilename) throws RemoteException {
		File xmlFile = new File(this.getMountFile().getParent()+File.separator+xmlFilename);
		try {
			if (!this.getMountFile().exists()) 
				throwException("round robine database doesn't exist or was removed", null);
			this.lockWrite();
			if ((!xmlFile.exists()) || (xmlFile.exists() && this.getMountFile().lastModified() > xmlFile.lastModified())) {
				int cmdSize = 4;
				String cmd[] = new String[cmdSize];			
				cmd[0] = RRDTOOL_CMD_BAT;
				cmd[1] = "dump";
				cmd[2] = this.getAbsolutePath();
				cmd[3] = xmlFile.getAbsolutePath();
				Process p = Runtime.getRuntime().exec(cmd);
				// Rcupration de son flot d'erreur
				InputStream err = p.getErrorStream();
				// Attente de la fin de son excution
				p.waitFor();
				// Lecture du flot d'erreur
				byte[] tabByte = new byte[err.available()];
				err.read(tabByte);
				err.close();
				String erreur = new String(tabByte);
				// Cas d'une erreur
				if (erreur.length() != 0) {
					throwException(erreur, null);
				}
			}				
			this.unLockWrite();
		} catch(Exception e) {
			throwRemoteException("Error dumping RoundRobinDataBase "+this.getMountFile().getName(), e);
		}
		return xmlFile;
	}
	/**
	* restore the contents of an RRD from its XML dump format
	* @param xmlFilename the name of the input xlm file 
	*/
	public RoundRobinDataBaseItf restore(String xmlFilename) throws RemoteException {
		File xmlFile = new File(this.getMountFile().getParent()+File.separator+xmlFilename);
		try {
			if (!this.getMountFile().exists()) 
				throwException("round robine database doesn't exist or was removed", null);
			if (!xmlFile.exists()) 
				throwException("the xml file specified doesn't exist or was removed", null);
			this.lockWrite();
			String cmd[] = new String[4];			
			cmd[0] = RRDTOOL_CMD;
			cmd[1] = "restore";
			cmd[2] = xmlFile.getAbsolutePath();
			cmd[3] = this.getAbsolutePath();
			rrdToolCmd(cmd);
			this.unLockWrite();
		} catch(Exception e) {
			e.printStackTrace();
			throwRemoteException("Error restoring RoundRobinDataBase "+this.getMountFile().getName()+" from "+xmlFile.getName(), e);
		}
		return (RoundRobinDataBaseItf)this;
	}
	/**
	* grow used if you want to add extra rows to an RRA. The extra rows will be inserted as the rows that are oldest. 
	* @param rraNum the RRA you want to alter. You can find the number using rrdtool info.
	* @param rows the number of rows you want to add. 
	*/
	public void grow(int rraNum, int rows) throws RemoteException {
		try {
			if (!this.getMountFile().exists()) 
				throwException("round robine database doesn't exist or was removed", null);
			this.lockWrite();
			String cmd[] = new String[6];			
			cmd[0] = RRDTOOL_CMD;
			cmd[1] = "resize";
			cmd[2] = this.getAbsolutePath();
			cmd[3] = String.valueOf(rraNum);
			cmd[4] = "GROW";
			cmd[5] = String.valueOf(rows);
			rrdToolCmd(cmd);
			this.unLockWrite();
		} catch(Exception e) {
			e.printStackTrace();
			throwRemoteException("Error growing RoundRobinDataBase "+this.getMountFile().getName(), e);
		}
	}
	/**
	* shrink used if you want to remove rows from an RRA. The rows that will be removed are the oldest rows. 
	* @param rraNum the RRA you want to alter. You can find the number using rrdtool info.
	* @param rows the number of rows you want to remove. 
	*/
	public void shrink(int rraNum, int rows) throws RemoteException {
		try {
			if (!this.getMountFile().exists()) 
				throwException("round robine database doesn't exist or was removed", null);
			this.lockWrite();
			String cmd[] = new String[6];			
			cmd[0] = RRDTOOL_CMD;
			cmd[1] = "resize";
			cmd[2] = this.getAbsolutePath();
			cmd[3] = String.valueOf(rraNum);
			cmd[4] = "SHRINK";
			cmd[5] = String.valueOf(rows);
			rrdToolCmd(cmd);
			this.unLockWrite();
		} catch(Exception e) {
			e.printStackTrace();
			throwRemoteException("Error shrinking RoundRobinDataBase "+this.getMountFile().getName(), e);
		}
	}
	/**
	* info return info about a round robin database
	*/
	public String info() throws RemoteException {
		RoundRobinInfo rrdInfo = null;
		try{
			rrdInfo = createRoundRobinInfo();
		} catch(Exception e) {
			e.printStackTrace();
			throwRemoteException("Error getting info of RoundRobinDataBase "+this.getMountFile().getName(), e);
		}
		return rrdInfo.toString();
	}
	/**
	* Returns the version of the round robin
	*/
	public String getRoundRobineVersion() throws RemoteException {
		RoundRobinInfo rrdInfo = null;
		try{
			rrdInfo = createRoundRobinInfo();
		} catch(Exception e) {
			throwRemoteException("Error getting version of RoundRobinDataBase "+this.getMountFile().getName(), e);
		}
		return rrdInfo.getRoundRobineVersion();
	}
	/**
	* Returns the step of the round robin
	*/
	public int getStep() throws RemoteException {
		RoundRobinInfo rrdInfo = null;
		try{
			rrdInfo = createRoundRobinInfo();
		} catch(Exception e) {
			throwRemoteException("Error getting step of RoundRobinDataBase "+this.getMountFile().getName(), e);
		}
		return rrdInfo.getStep();
	}
	/**
	* Returns the data source container of the round robin
	*/
	public DataSourceContainer getDataSourceContainer() throws RemoteException {
		RoundRobinInfo rrdInfo = null;
		try{
			rrdInfo = createRoundRobinInfo();
		} catch(Exception e) {
			e.printStackTrace();
			throwRemoteException("Error getting DataSourceContainer of RoundRobinDataBase "+this.getMountFile().getName(), e);
		}
		
		return rrdInfo.getDataSourceContainer();
	}
	/**
	* Returns the round robin archive container of the round robin
	*/
	public RoundRobinArchiveContainer getRoundRobinArchiveContainer() throws RemoteException {
		RoundRobinInfo rrdInfo = null;
		try{
			rrdInfo = createRoundRobinInfo();
		} catch(Exception e) {
			throwRemoteException("Error getting RoundRobinArchiveContainer of RoundRobinDataBase "+this.getMountFile().getName(), e);
		}
		return rrdInfo.getRoundRobinArchiveContainer();
	}
	/**
	* createInfoFile create the file of info on this RRD
	*/
	private RoundRobinInfo createRoundRobinInfo() throws Exception {
		File infoFile = null;
		RoundRobinInfo rrdInfo = null;
		try {
			if (!this.getMountFile().exists()) 
				throwException("round robine database doesn't exist or was removed", null);
			this.lockWrite();
			infoFile = new File(this.getAbsolutePath().substring(0, this.getAbsolutePath().length()-3)+"info");
			if ((!infoFile.exists()) || (infoFile.exists() && this.getMountFile().lastModified() > infoFile.lastModified())) {
				int cmdSize = 4;
				String cmd[] = new String[cmdSize];			
				cmd[0] = RRDTOOL_CMD_BAT;
				cmd[1] = "info";
				cmd[2] = this.getAbsolutePath();
				cmd[3] = infoFile.getAbsolutePath();
				Process p = Runtime.getRuntime().exec(cmd);
				// Rcupration de son flot d'erreur
				InputStream err = p.getErrorStream();
				// Attente de la fin de son excution
				p.waitFor();
				// Lecture du flot d'erreur
				byte[] tabByte = new byte[err.available()];
				err.read(tabByte);
				err.close();
				String erreur = new String(tabByte);
				// Cas d'une erreur
				if (erreur.length() != 0) {
					throwException(erreur, null);
				}
			}				
			rrdInfo = new RoundRobinInfo(infoFile);
			System.out.println(rrdInfo.toString());
			this.unLockWrite();
		} catch(Exception e) {
			e.printStackTrace();
			throwException("Error creating RoundRobineInfo of RoundRobinDataBase", e);
		}
		return rrdInfo;
	}
	/**
	* getSize return the size of RoundRobinDataBase
	* @return long
	*/
	public long getSize() throws RemoteException {
		long size = 0;
		try {
			this.lockRead();
			size = this.getMountFile().length();
			this.unLockRead();
		} catch(Exception e) {
			throwRemoteException("Error getting size of RoundRobinDataBase "+this.getMountFile().getName(), e);
		}
		return size;
	}
	/**
	* graph Create a graph based on data from one or several RRD
	* @param roundRobinGraphic the roundRobinGraphic which represent the graph
	* @return Image the file of the image
	*/
	public RoundRobinImage graph(RoundRobinGraphic roundRobinGraphic) throws RemoteException { 
		File imageFile = null;
		RoundRobinImage graph = null;
		try {
			if (roundRobinGraphic.getImageFormat() == null || roundRobinGraphic.getImageFormat().equals(roundRobinGraphic.GIF_FORMAT)) {
				imageFile = new File(this.getMountFile().getParent()+File.separator+roundRobinGraphic.getGraphicName()+".gif");
			}
			else {
				imageFile = new File(this.getMountFile().getParent()+File.separator+roundRobinGraphic.getGraphicName()+".png");
			}
			if (imageFile.exists()) {
				imageFile.delete();
			}
			if (!this.getMountFile().exists()) {
				throwException("round robine database doesn't exist or was removed", null);
			}
			this.lockRead();
			String cmd = new String(RRDTOOL_CMD+" graph "+imageFile.getName()+" "+roundRobinGraphic.toString());
			System.out.println(cmd);
			mutex.P();
			Process p = Runtime.getRuntime().exec(cmd, null, this.getMountFile().getParentFile());
			// Rcupration de son flot d'erreur
			InputStream err = p.getErrorStream();
			// Attente de la fin de son excution
			p.waitFor();
			// Lecture du flot d'erreur
			byte[] tabByte = new byte[err.available()];
			err.read(tabByte);
			err.close();
			String erreur = new String(tabByte);
			mutex.V();
			// Cas d'une erreur
			if (erreur.length() != 0) {
				throwException(erreur, null);
			}				
			this.unLockRead();
			graph = new RoundRobinImage(imageFile);
		} catch(Exception e) {
			throwRemoteException("Error creating graph of RoundRobinDataBase", e);
		}
		return graph;
	}
	/**
	* delete delete a rrd
	* @return int the number of rrd deleted
	*/
	public int delete() throws RemoteException {
		try {
			this.lockDelete();
			this.getMountFile().delete();
			RoundRobinService.lockTable.removeLock(this.getMountFile().getAbsolutePath());
		} catch(Exception e) {
			throwRemoteException("Error deleting RoundRobinDataBase "+this.getMountFile().getName(), e);
		} 
		return 1;		
	}
	/**
	* rrdToolCmd execute un rrdtool commande
	* @param commande: the commande to execute
	*/
	private String rrdToolCmd(String[] cmd) throws Exception {
		String result = new String();
		try {
			// Affichage de la commande
			//printCmd(cmd);
			// Lancement du thread avec le mme environnemt que le pre dans le rpertoire de travail outputFolder
			Process p = Runtime.getRuntime().exec(cmd , null, this.getMountFile().getParentFile());
			// Rcupration de son flot d'erreur
			InputStream err = p.getErrorStream();
			// Rcupration de son flot de sortie
			InputStream in = p.getInputStream();
			// Attente de la fin de son excution
			p.waitFor();
			// Lecture du flot d'erreur
			byte[] tabByte = new byte[err.available()];
			err.read(tabByte);
			err.close();
			String erreur = new String(tabByte);
			// Cas d'une erreur
			if (erreur.length() != 0) {
				throwException(erreur, null);
			}
			// Lecture du flot de sortie
			tabByte = new byte[in.available()];
			in.read(tabByte);
			in.close();
			result = new String(tabByte);
		} catch (Exception e) {
			e.printStackTrace();
			//throwException("Error commande rrdtool "+cmd[1], e);
		}
		return result;
	}
	/**
	* printCmd display a command line
	* @param cmd: command line to display
	*/
	private void printCmd(String[] cmd) {
		for (int i=0; i<cmd.length; i++) {
			System.out.print(cmd[i]+" ");
		}
		System.out.println("");
	}
}
