package rrdService.service;
import rrdService.util.*;
import java.rmi.*;
import java.io.*;
import java.lang.*;
/**	
* The class RoundRobinInfo allows to known all info about a RoundRobin
* @author Pellier Damien FIRMENICH SA
* @version	1.0
* date : 18/06/01
* </code>
* @see <a href="RoundRobinInfo.java">RoundRobinInfo</a>
* @see RoundRobin RoundRobinDataBase RoundRobinLibrary
*/
public class RoundRobinInfo {
	/**
	* The file with info
	*/ 
	private FileInputStream infoFileInputStream;
	/**
	* filename of the round robine
	*/
	private String roundRobinName;
	/**
	* round robine version
	*/
	private String roundRobinVersion;
	/**
	* step of the round robin
	*/
	private int step;
	/**
	* last update of the round robin
	*/
	private long lastUpdate;
	/**
	* data source container of the round robine
	*/
	private DataSourceContainer DSContainer;
	/**
	* round robine archive container of the round robine
	*/
	private RoundRobinArchiveContainer RRAContainer;
	/**
	* Constructor of a RoundRobinInfo
	*/
	public RoundRobinInfo(File infoFile) throws Exception {	
		boolean read = true;
		String line = new String();
		try {
			this.DSContainer = new DataSourceContainer();
			this.RRAContainer = new RoundRobinArchiveContainer();
			if (!infoFile.exists()) throw new Exception(infoFile.getName()+" doesn\'t exist");
			this.infoFileInputStream = new FileInputStream(infoFile);
			
			while(notEndOfFile()) {
				if (read) line = readLine();
				String lexem = readLexem(line);
				if (lexem.equals("filename")) {
					this.roundRobinName = readValue(line);
					read = true;
				}
				else if (lexem.equals("rrd_version")) {
					this.roundRobinVersion = readValue(line);
					this.roundRobinVersion = this.roundRobinVersion.substring(1, this.roundRobinVersion.length()-1);
					read = true;
				}
				else if (lexem.equals("step")) {
					this.step = Integer.parseInt(readValue(line));
					read = true;
				}
				else if (lexem.equals("last_update")) {
					this.lastUpdate = Long.parseLong(readValue(line));
					read = true;
				}
				else if (lexem.substring(0, 2).equals("ds")) {
					DataSource ds = new DataSource();
					ds.setDsName(readDsName(lexem)); 
					ds.setDataSourceType(readValue(line).substring(1, readValue(line).length()-1));
					ds.setHeartbeat(Integer.parseInt(readValue(readLine())));
					ds.setMin(readDoubleValue(readLine()).doubleValue());
					ds.setMax(readDoubleValue(readLine()).doubleValue());
					ds.setLastDataSource(readValue(readLine()));
					ds.setValue(readDoubleValue(readLine()).doubleValue());
					ds.setUnknownSec(Integer.parseInt(readValue(readLine())));
					DSContainer.addElement(ds);	
				}
				else if (lexem.substring(0, 3).equals("rra")) {
					RoundRobinArchive rra = new RoundRobinArchive(); 
					rra.setConsolidationFunction(readValue(line).substring(1, readValue(line).length()-1));
					rra.setRows(Integer.parseInt(readValue(readLine())));
					rra.setSteps(Integer.parseInt(readValue(readLine())));
					rra.setXFilesFactor(readDoubleValue(readLine()).doubleValue());
					CdpPrepContainer container = new CdpPrepContainer();
					String l = readLine();
					String oldRra = readLexem(l).substring(0, 6);
					String newRra = new String(oldRra);
					while (notEndOfFile() && oldRra.equals(newRra)) {
						CdpPrep cdpPrep = new CdpPrep();
						cdpPrep.setValue(readDoubleValue(l));
						l = readLine();
						cdpPrep.setUnknonDataPoints(Integer.parseInt(readValue(l)));	
						container.addElement(cdpPrep);
						oldRra = readLexem(l).substring(0, 6);
						l = readLine();
						if (notEndOfFile()) newRra = readLexem(l).substring(0, 6);
					}
					rra.setCdpPrepContainer(container);
					RRAContainer.addElement(rra);
					read = false;
					line = l;
				}
				else {
					throw new Exception();
				}
			}
			infoFileInputStream.close();
		} catch(Exception e) {
			e.printStackTrace();
			throw new Exception();
		}
	}
	/**
	* Returns the name of the round robin
	*/
	public String getRoundRobinName() {
		return this.roundRobinName;
	}
	/**
	* Returns the version of the round robin
	*/
	public String getRoundRobineVersion() {
		return this.roundRobinVersion;
	}
	/**
	* Returns the step of the round robin
	*/
	public int getStep() {
		return this.step;
	}
	/**
	* Returns the last update of the round robin
	*/
	public long getLastUpdate() {
		return this.lastUpdate;
	}
	/**
	* Returns the data source container of the round robin
	*/
	public DataSourceContainer getDataSourceContainer() {
		return this.DSContainer;
	}
	/**
	* Returns the round robin archive container of the round robin
	*/
	public RoundRobinArchiveContainer getRoundRobinArchiveContainer() {
		return this.RRAContainer;
	}
	/**
	* toString return a string representation of a RRDInfo
	*/
	public String toString() {
		return 	"filename = "+roundRobinName+"\n"+
				"rrd_version = "+roundRobinVersion+"\n"+
				"step = "+String.valueOf(step)+"\n"+
				"last_update = "+String.valueOf(lastUpdate)+"\n"+
				DSContainer.toString()+"\n"+
				RRAContainer.toString();
	}
	/**
	* notEndOfFile return true if end of config file is reach
	*/
	private boolean notEndOfFile() throws Exception {
		return infoFileInputStream.available() > 0;
	}
	/**
	* readLexem read a lexem
	* @param line the line to test
	*/
	private String readLexem(String line) {
		String lexem = new String();
		int i = 0;
		while(i<line.length() && line.charAt(i) != ' ') { 
			lexem += line.charAt(i);
			i++;
		}
		return lexem;
	}
	/**
	* readDsName read the DsName of a DS line
	* @param lexem the lexem to test
	*/
	private String readDsName(String lexem) {
		String dsName = new String();
		int i = 0;
		while(i<lexem.length() && lexem.charAt(i) != '[') i++;
		i++;
		while(i<lexem.length() && lexem.charAt(i) != ']') {
			dsName+= lexem.charAt(i);
			i++;
		}
		return dsName;
	}
	/**
	* readLine read a line
	*/
	private String readLine() throws Exception {
		char c;
		String line = new String();
		while(notEndOfFile() && ((c = (char)infoFileInputStream.read()) != Character.LINE_SEPARATOR)) {
			line += c;
		}
		if (notEndOfFile()) {
			infoFileInputStream.read();
		}
		return line;
	}
	/**
	* readValue read the value of a lexem
	* @param line the line to test
	*/
	private String readValue(String line) throws Exception {
		String value = new String();
		int i = 0;
		while(i<line.length() && line.charAt(i) != '=') i++;
		i += 2;
		while(i<line.length()) {
			value+= line.charAt(i);
			i++;
		}
		return value;
	}
	/**
	* readDoubleValue read the value of a lexem which is a double
	* @param line the line to test
	*/
	private Double readDoubleValue(String line) throws Exception {
		String value = readValue(line);
		if (value.equals("NaN")) {
			return null;
		}
		String number = new String();
		String exposant = new String();
		int i = 0;
		while(i<value.length() && value.charAt(i) != 'e') {
			number+= value.charAt(i);
			i++;
		}
		i++;
		while(i<value.length()) {
			exposant+= value.charAt(i);
			i++;
		}
		char signe = exposant.charAt(0);
		exposant = exposant.substring(1, exposant.length());
		Double result = null;
		if (signe == '-') 
			result = new Double(Double.parseDouble(number)/(10*Double.parseDouble(exposant)));
		else
			result = new Double(Double.parseDouble(number)*(10*Double.parseDouble(exposant)));
		return result;
	}		
}